@@ -1,16 +1,15 @@
-from __future__ import absolute_import
-
 from importlib import import_module
 
 from django import forms
 from django.contrib.auth import password_validation
 from django.contrib.auth.tokens import PasswordResetTokenGenerator
-from django.contrib.sites.shortcuts import get_current_site
 from django.core import exceptions, validators
-from django.urls import reverse
+from django.urls import NoReverseMatch, reverse
 from django.utils.safestring import mark_safe
 from django.utils.translation import gettext, gettext_lazy as _, pgettext
 
+from allauth.account.authentication import record_authentication
+
 from ..utils import (
     build_absolute_uri,
     get_username_max_length,
@@ -108,9 +107,8 @@ class LoginForm(forms.Form):
         self.request = kwargs.pop("request", None)
         super(LoginForm, self).__init__(*args, **kwargs)
         if app_settings.AUTHENTICATION_METHOD == AuthenticationMethod.EMAIL:
-            login_widget = forms.TextInput(
+            login_widget = forms.EmailInput(
                 attrs={
-                    "type": "email",
                     "placeholder": _("Email address"),
                     "autocomplete": "email",
                 }
@@ -140,11 +138,15 @@ class LoginForm(forms.Form):
         set_form_field_order(self, ["login", "password", "remember"])
         if app_settings.SESSION_REMEMBER is not None:
             del self.fields["remember"]
-        self.fields["password"].help_text = mark_safe(
-            _('<a href="{}">Forgot your password?</a>').format(
-                reverse("account_reset_password")
+        try:
+            reset_url = reverse("account_reset_password")
+        except NoReverseMatch:
+            pass
+        else:
+            forgot_txt = _("Forgot your password?")
+            self.fields["password"].help_text = mark_safe(
+                f'<a href="{reset_url}">{forgot_txt}</a>'
             )
-        )
 
     def user_credentials(self):
         """
@@ -198,13 +200,19 @@ class LoginForm(forms.Form):
         return self.cleaned_data
 
     def login(self, request, redirect_url=None):
-        email = self.user_credentials().get("email")
+        credentials = self.user_credentials()
+        extra_data = {
+            field: credentials.get(field)
+            for field in ["email", "username"]
+            if field in credentials
+        }
+        record_authentication(request, method="password", **extra_data)
         ret = perform_login(
             request,
             self.user,
             email_verification=app_settings.EMAIL_VERIFICATION,
             redirect_url=redirect_url,
-            email=email,
+            email=credentials.get("email"),
         )
         remember = app_settings.SESSION_REMEMBER
         if remember is None:
@@ -443,14 +451,15 @@ class SignupForm(BaseSignupForm):
         return self.cleaned_data
 
     def save(self, request):
+        email = self.cleaned_data.get("email")
         if self.account_already_exists:
-            raise ValueError(self.cleaned_data.get("email"))
+            raise ValueError(email)
         adapter = get_adapter()
         user = adapter.new_user(request)
         adapter.save_user(request, user, self)
         self.custom_signup(request, user)
         # TODO: Move into adapter `save_user` ?
-        setup_user_email(request, user, [])
+        setup_user_email(request, user, [EmailAddress(email=email)] if email else [])
         return user
 
 
@@ -579,7 +588,8 @@ class ResetPasswordForm(forms.Form):
     def save(self, request, **kwargs):
         email = self.cleaned_data["email"]
         if not self.users:
-            self._send_unknown_account_mail(request, email)
+            if app_settings.EMAIL_UNKNOWN_ACCOUNTS:
+                self._send_unknown_account_mail(request, email)
         else:
             self._send_password_reset_mail(request, email, self.users, **kwargs)
         return email
@@ -587,7 +597,6 @@ class ResetPasswordForm(forms.Form):
     def _send_unknown_account_mail(self, request, email):
         signup_url = build_absolute_uri(request, reverse("account_signup"))
         context = {
-            "current_site": get_current_site(request),
             "email": email,
             "request": request,
             "signup_url": signup_url,
@@ -613,7 +622,6 @@ class ResetPasswordForm(forms.Form):
             url = build_absolute_uri(request, path)
 
             context = {
-                "current_site": get_current_site(request),
                 "user": user,
                 "password_reset_url": url,
                 "uid": uid,
@@ -686,7 +694,7 @@ class ReauthenticateForm(forms.Form):
 
     def clean_password(self):
         password = self.cleaned_data.get("password")
-        if not self.user.check_password(password):
+        if not get_adapter().reauthenticate(self.user, password):
             raise forms.ValidationError(
                 get_adapter().error_messages["incorrect_password"]
             )
