@@ -310,7 +310,7 @@ def cleanup_email_addresses(request, addresses):
         primary_address = primary_addresses[0]
     elif e2a:
         # Pick the first
-        primary_address = e2a.keys()[0]
+        primary_address = list(e2a.values())[0]
     else:
         # Empty
         primary_address = None
@@ -366,25 +366,40 @@ def send_email_confirmation(request, user, signup=False, email=None):
     sent (consider a user retrying a few times), which is why there is
     a cooldown period before sending a new mail. This cooldown period
     can be configured in ACCOUNT_EMAIL_CONFIRMATION_COOLDOWN setting.
+
+    TODO: This code is doing way too much. Looking up EmailAddress, creating
+    if not present, etc. To be refactored.
     """
     from .models import EmailAddress
 
     adapter = get_adapter()
 
+    email_address = None
     if not email:
         email = user_email(user)
+    if not email:
+        email_address = (
+            EmailAddress.objects.filter(user=user).order_by("verified", "pk").first()
+        )
+        if email_address:
+            email = email_address.email
+
     if email:
-        try:
-            email_address = EmailAddress.objects.get_for_user(user, email)
+        if email_address is None:
+            try:
+                email_address = EmailAddress.objects.get_for_user(user, email)
+            except EmailAddress.DoesNotExist:
+                pass
+        if email_address is not None:
             if not email_address.verified:
                 send_email = adapter.should_send_confirmation_mail(
-                    request, email_address
+                    request, email_address, signup
                 )
                 if send_email:
                     email_address.send_confirmation(request, signup=signup)
             else:
                 send_email = False
-        except EmailAddress.DoesNotExist:
+        else:
             send_email = True
             email_address = EmailAddress.objects.add_email(
                 request, user, email, signup=signup, confirm=True
