@@ -1,5 +1,4 @@
-from __future__ import absolute_import
-
+import functools
 import warnings
 
 from django.core.exceptions import (
@@ -230,7 +229,10 @@ class DefaultSocialAccountAdapter(object):
         provider_to_apps = {}
 
         # First, populate it with the DB backed apps.
-        db_apps = SocialApp.objects.on_site(request)
+        if request:
+            db_apps = SocialApp.objects.on_site(request)
+        else:
+            db_apps = SocialApp.objects.all()
         if provider:
             db_apps = db_apps.filter(Q(provider=provider) | Q(provider_id=provider))
         if client_id:
@@ -290,6 +292,15 @@ class DefaultSocialAccountAdapter(object):
             raise SocialApp.DoesNotExist()
         return apps[0]
 
+    def get_requests_session(self):
+        import requests
+
+        session = requests.Session()
+        session.request = functools.partial(
+            session.request, timeout=app_settings.REQUESTS_TIMEOUT
+        )
+        return session
+
 
 def get_adapter(request=None):
     return import_attribute(app_settings.ADAPTER)(request)
