@@ -1,7 +1,7 @@
-import requests
 import xml.etree.ElementTree as ET
 
 from allauth.socialaccount import app_settings
+from allauth.socialaccount.adapter import get_adapter
 from allauth.socialaccount.providers.oauth2.views import (
     OAuth2Adapter,
     OAuth2CallbackView,
@@ -25,7 +25,11 @@ class NextCloudAdapter(OAuth2Adapter):
 
     def get_user_info(self, token, user_id):
         headers = {"Authorization": "Bearer {0}".format(token)}
-        resp = requests.get(self.profile_url + user_id, headers=headers)
+        resp = (
+            get_adapter()
+            .get_requests_session()
+            .get(self.profile_url + user_id, headers=headers)
+        )
         resp.raise_for_status()
         data = ET.fromstring(resp.content.decode())[1]
         return {d.tag: d.text.strip() for d in data if d.text is not None}
